'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

var BINARY_TREE_TYPE = {
    BALANCED: 'balanced',
    COMPLETE: 'complete',
    PERFECT: 'perfect',
    FULL: 'full',
};
var TREE_ID = 'react-tree-vis';
var CSS_VARIABLE_MAP = {
    lineColor: {
        variableName: '--line-color',
        defaultValue: '#ccc',
    },
    lineHoverColor: {
        variableName: '--line-hover-color',
        defaultValue: '#5f6674',
    },
    lineRadius: {
        variableName: '--line-radius',
        defaultValue: '5px',
    },
    nodeBorder: {
        variableName: '--node-border',
        defaultValue: 'none',
    },
    nodeBorderRadius: {
        variableName: '--node-border-radius',
        defaultValue: '200px',
    },
    nodeBackgroundColor: {
        variableName: '--node-bg-color',
        defaultValue: '#fff',
    },
    nodeFontColor: {
        variableName: '--node-font-color',
        defaultValue: '#666',
    },
    nodeShadow: {
        variableName: '--node-shadow',
        defaultValue: '-5px -5px 20px #fff, 5px 5px 20px #babecc',
    },
    nodeTextShadow: {
        variableName: '--node-text-shadow',
        defaultValue: 'none',
    },
    nodeHighlightBorder: {
        variableName: '--node-hl-border',
        defaultValue: 'none',
    },
    nodeHighlightBackgroundColor: {
        variableName: '--node-hl-bg-color',
        defaultValue: '#fff',
    },
    nodeHighlightFontColor: {
        variableName: '--node-hl-font-color',
        defaultValue: '#fff',
    },
    nodeHighlightShadow: {
        variableName: '--node-hl-shadow',
        defaultValue: '-5px -5px 20px #fff, 5px 5px 20px #babecc',
    },
    nodeHighlightTextShadow: {
        variableName: '--node-hl-text-shadow',
        defaultValue: '0 0 5px #fff, 0 0 10px #fff, 0 0 15px #00ff15, 0 0 20px #00ff15, 0 0 25px #00ff15, 0 0 30px #00ff15, 0 0 35px #00ff15',
    },
    nodeHoverBorder: {
        variableName: '--node-hover-border',
        defaultValue: 'none',
    },
    nodeHoverBackgroundColor: {
        variableName: '--node-hover-bg-color',
        defaultValue: '#fff',
    },
    nodeHoverFontColor: {
        variableName: '--node-hover-font-color',
        defaultValue: '#002574',
    },
    nodeHoverShadow: {
        variableName: '--node-hover-shadow',
        defaultValue: '-1px -1px 5px #fff, 1px 1px 5px #babecc',
    },
    nodeHoverTextShadow: {
        variableName: '--node-hover-text-shadow',
        defaultValue: 'none',
    },
    nodeNullFontColor: {
        variableName: '--node-null-font-color',
        defaultValue: '#7c7c7c2f',
    },
    nodeNullHoverFontColor: {
        variableName: '--node-null-hover-font-color',
        defaultValue: '#ff0000b9',
    },
    transitionDuration: {
        variableName: '--transition',
        defaultValue: '0.5s',
    },
};

var LEFT = 'left';
var RIGHT = 'right';
var NULL_NODE_JSX = (React__default['default'].createElement("li", { className: "null" },
    React__default['default'].createElement("div", null, "null")));
var getNormalNodeJSX = function (value, leftJSX, rightJSX, isHighlighted) {
    if (isHighlighted === void 0) { isHighlighted = false; }
    return (React__default['default'].createElement("li", { key: value },
        React__default['default'].createElement("div", { className: "normal " + (isHighlighted ? 'highlight' : '') }, value),
        React__default['default'].createElement("ul", null,
            leftJSX,
            " ",
            rightJSX)));
};
var Node = /** @class */ (function () {
    function Node(value) {
        this.value = value;
        this.left = null;
        this.right = null;
        this.parent = null;
        this.leftJSX = NULL_NODE_JSX;
        this.rightJSX = NULL_NODE_JSX;
        this.currentJSX = getNormalNodeJSX(this.value, this.leftJSX, this.rightJSX);
    }
    //Inserts a node in JSX
    Node.prototype.insert = function (node, isLeft) {
        var newJSX = node.currentJSX;
        if (isLeft) {
            this.leftJSX = newJSX;
        }
        else {
            this.rightJSX = newJSX;
        }
        this.setJSX();
        this.updateRootJSX();
    };
    //Updates JSX for the node
    Node.prototype.setJSX = function () {
        this.currentJSX = getNormalNodeJSX(this.value, this.leftJSX, this.rightJSX);
    };
    //Updates the whole JSX
    Node.prototype.updateRootJSX = function () {
        if (this.parent !== null) {
            if (this.parent.left === this)
                this.parent.insert(this, true);
            else
                this.parent.insert(this, false);
        }
    };
    //Sets one child to null
    Node.prototype.setChildToNull = function (isLeft) {
        if (isLeft)
            this.leftJSX = NULL_NODE_JSX;
        else
            this.rightJSX = NULL_NODE_JSX;
        this.setJSX();
        this.updateRootJSX();
    };
    //Sets grandchild to child
    Node.prototype.setChildToChildsChild = function (isLeftChild, isLeft) {
        if (isLeftChild) {
            if (isLeft) {
                this.leftJSX = this.left.left.currentJSX;
                this.setJSX();
            }
            else {
                this.leftJSX = this.left.right.currentJSX;
                this.setJSX();
            }
        }
        else {
            if (isLeft) {
                this.rightJSX = this.right.left.currentJSX;
                this.setJSX();
            }
            else {
                this.rightJSX = this.right.right.currentJSX;
                this.setJSX();
            }
        }
        this.updateRootJSX();
    };
    //Removes a node from JSX
    Node.prototype.remove = function (childrenCondtion) {
        if (!this.parent)
            return;
        if (!childrenCondtion) {
            if (this.parent.left === this) {
                this.parent.setChildToNull(true);
            }
            else {
                this.parent.setChildToNull(false);
            }
        }
        else if (childrenCondtion === LEFT) {
            if (this.parent.left === this) {
                this.parent.setChildToChildsChild(true, true);
            }
            else {
                this.parent.setChildToChildsChild(false, true);
            }
        }
        else {
            if (this.parent.left === this) {
                this.parent.setChildToChildsChild(true, false);
            }
            else {
                this.parent.setChildToChildsChild(false, false);
            }
        }
    };
    //Updates the value of a node
    Node.prototype.updateValue = function (value) {
        this.value = value;
        this.setJSX();
        this.updateRootJSX();
    };
    //Adds highlight to a node when searched
    Node.prototype.addHighlight = function () {
        this.currentJSX = getNormalNodeJSX(this.value, this.leftJSX, this.rightJSX, true);
        this.updateRootJSX();
    };
    //Removes the highlight
    Node.prototype.clearHighlight = function () {
        this.setJSX();
        this.updateRootJSX();
    };
    //Changes the JSX when tree is balanced
    Node.prototype.balanceJSX = function () {
        if (!this.left && this.right) {
            this.rightJSX = this.right.currentJSX;
            this.setChildToNull(true);
        }
        else if (!this.right && this.left) {
            this.leftJSX = this.left.currentJSX;
            this.setChildToNull(false);
        }
        else if (this.right && this.left) {
            this.leftJSX = this.left.currentJSX;
            this.rightJSX = this.right.currentJSX;
            this.setJSX();
            this.updateRootJSX();
        }
        else {
            this.leftJSX = NULL_NODE_JSX;
            this.rightJSX = NULL_NODE_JSX;
            this.setJSX();
            this.updateRootJSX();
        }
    };
    return Node;
}());
var BST = /** @class */ (function () {
    function BST(num) {
        if (num === void 0) { num = 0; }
        this.root = null;
        if (num) {
            this.generateRandomBST(num);
        }
        this.highlightedNode = null;
        this.leafDepth = -1;
    }
    //inserting a new node
    BST.prototype.insert = function (value) {
        this.clearHighlight();
        var newNode = new Node(value);
        if (this.root === null) {
            this.root = newNode;
        }
        else
            this.insertNode(this.root, newNode);
    };
    BST.prototype.insertNode = function (node, newNode) {
        if (newNode.value < node.value) {
            if (node.left === null) {
                node.left = newNode;
                newNode.parent = node;
                node.insert(newNode, true);
            }
            else
                this.insertNode(node.left, newNode);
        }
        else {
            if (node.right === null) {
                node.right = newNode;
                newNode.parent = node;
                node.insert(newNode, false);
            }
            else
                this.insertNode(node.right, newNode);
        }
    };
    //Finding max value
    BST.prototype.findMax = function (node) {
        if (node === void 0) { node = this.root; }
        if (node == null)
            return node;
        else if (node.right == null) {
            return node;
        }
        else {
            return this.findMax(node.right);
        }
    };
    //Finding min value
    BST.prototype.findMin = function (node) {
        if (node === void 0) { node = this.root; }
        if (node == null)
            return node;
        else if (node.left == null) {
            return node;
        }
        else {
            return this.findMin(node.left);
        }
    };
    //Deleting node
    BST.prototype.remove = function (value) {
        this.clearHighlight();
        this.root = this.removeNode(this.root, value);
    };
    // TODO: Probably returning a node is not required, handling it simialr to insert function
    BST.prototype.removeNode = function (node, value) {
        if (node === null)
            return null;
        else if (value < node.value) {
            node.left = this.removeNode(node.left, value);
            return node;
        }
        else if (value > node.value) {
            node.right = this.removeNode(node.right, value);
            return node;
        }
        else {
            if (node.left === null && node.right === null) {
                node.remove(null);
                node = null;
                return node;
            }
            if (node.left === null) {
                node.remove(RIGHT);
                node.right.parent = node.parent;
                node = node.right;
                return node;
            }
            else if (node.right === null) {
                node.remove(LEFT);
                node.left.parent = node.parent;
                node = node.left;
                return node;
            }
            var minNodeOfRight = this.findMin(node.right);
            node.updateValue(minNodeOfRight.value);
            node.right = this.removeNode(node.right, minNodeOfRight.value);
            return node;
        }
    };
    //Find height
    BST.prototype.height = function (node) {
        if (node === void 0) { node = this.root; }
        if (node === null)
            return -1;
        var ans;
        ans = this.height(node.left);
        ans = Math.max(this.height(node.right), ans);
        return ans + 1;
    };
    //Count number of nodes
    BST.prototype.countNodes = function (node) {
        if (node === void 0) { node = this.root; }
        if (node === null)
            return 0;
        return this.countNodes(node.left) + 1 + this.countNodes(node.right);
    };
    //BST traversal
    BST.prototype.preorder = function (list, node) {
        if (node === void 0) { node = this.root; }
        if (node !== null) {
            list.push(node.value);
            this.preorder(list, node.left);
            this.preorder(list, node.right);
        }
    };
    BST.prototype.inorder = function (list, node) {
        if (node === void 0) { node = this.root; }
        if (node !== null) {
            this.inorder(list, node.left);
            list.push(node.value);
            this.inorder(list, node.right);
        }
    };
    BST.prototype.postorder = function (list, node) {
        if (node === void 0) { node = this.root; }
        if (node !== null) {
            this.postorder(list, node.left);
            this.postorder(list, node.right);
            list.push(node.value);
        }
    };
    //Search
    BST.prototype.search = function (value, node) {
        if (node === void 0) { node = this.root; }
        this.clearHighlight();
        if (node === null)
            return false;
        else if (node.value === value) {
            node.addHighlight();
            this.highlightedNode = node;
            return true;
        }
        else if (node.value > value)
            return this.search(value, node.left);
        else
            return this.search(value, node.right);
    };
    //Random Tree generator
    BST.prototype.generateRandomBST = function (num) {
        var upper = 0;
        var lower = num * 2 + 10;
        var elements = new Set();
        for (var i = 0; i < num; i++) {
            var value = Math.floor(Math.random() * (upper - lower + 1)) + lower;
            while (elements.has(value)) {
                value = Math.floor(Math.random() * (upper - lower + 1)) + lower;
            }
            elements.add(value);
            this.insert(value);
        }
    };
    //BST Checker
    //Main Checker function
    BST.prototype.checkBST = function () {
        var BALANCED = BINARY_TREE_TYPE.BALANCED, COMPLETE = BINARY_TREE_TYPE.COMPLETE, FULL = BINARY_TREE_TYPE.FULL, PERFECT = BINARY_TREE_TYPE.PERFECT;
        var BSTTypeList = [];
        if (this.isBalanced()) {
            BSTTypeList.push(BALANCED);
        }
        if (this.isComplete()) {
            BSTTypeList.push(COMPLETE);
        }
        if (this.isPerfect()) {
            BSTTypeList.push(FULL);
        }
        if (this.isFull()) {
            BSTTypeList.push(PERFECT);
        }
        return BSTTypeList;
    };
    //Subfunctions
    //balance
    BST.prototype.isBalanced = function (node) {
        if (node === void 0) { node = this.root; }
        if (node === null)
            return true;
        var leftHeight = this.height(node.left);
        var rightHeight = this.height(node.right);
        if (Math.abs(leftHeight - rightHeight) <= 1) {
            return this.isBalanced(node.left) && this.isBalanced(node.right);
        }
        else {
            return false;
        }
    };
    //complete
    BST.prototype.isComplete = function () {
        var totalNodes = this.countNodes();
        return this.isComplete2(0, totalNodes);
    };
    BST.prototype.isComplete2 = function (index, totalNodes, node) {
        if (node === void 0) { node = this.root; }
        if (node === null)
            return true;
        else if (index >= totalNodes)
            return false;
        else {
            var ans = this.isComplete2(index * 2 + 1, totalNodes, node.left);
            ans && (ans = this.isComplete2(index * 2 + 2, totalNodes, node.right));
            return ans;
        }
    };
    //perfect
    BST.prototype.isPerfect = function () {
        if (!this.isFull() || !this.isComplete())
            return false;
        return this.isPerfect2();
    };
    BST.prototype.isPerfect2 = function (node, depth) {
        if (node === void 0) { node = this.root; }
        if (depth === void 0) { depth = 0; }
        if (!node)
            return true;
        depth++;
        if (!node.right && !node.left) {
            if (this.leafDepth !== -1 && this.leafDepth !== depth)
                return false;
            this.leafDepth = depth;
        }
        return (this.isPerfect2(node.left, depth) && this.isPerfect2(node.right, depth));
    };
    //full
    BST.prototype.isFull = function (node) {
        if (node === void 0) { node = this.root; }
        if (node === null)
            return true;
        else if (node.left === null && node.right === null)
            return true;
        else if (node.left !== null && node.right !== null) {
            var ans = void 0;
            ans = this.isFull(node.left);
            ans && (ans = this.isFull(node.right));
            return ans;
        }
        else
            return false;
    };
    //Balancing BST
    // TODO: improve the algorithm, it does not work for repeated values
    BST.prototype.balance = function (node) {
        if (node === void 0) { node = this.root; }
        this.clearHighlight();
        var nodes = [];
        this.BSTToArr(node, nodes);
        var length = nodes.length;
        this.root = null;
        this.makeBST(nodes, 0, length - 1);
    };
    //Balancing sub functions
    BST.prototype.BSTToArr = function (node, nodes) {
        if (node !== null) {
            this.BSTToArr(node.left, nodes);
            nodes.push(node);
            this.BSTToArr(node.right, nodes);
        }
    };
    //making BST from array
    BST.prototype.makeBST = function (nodes, start, end) {
        if (start > end) {
            return;
        }
        var mid = Math.floor((start + end) / 2);
        this.insert(nodes[mid].value);
        this.makeBST(nodes, start, mid - 1);
        this.makeBST(nodes, mid + 1, end);
    };
    BST.prototype.clearHighlight = function () {
        if (this.highlightedNode) {
            this.highlightedNode.clearHighlight();
            this.highlightedNode = null;
        }
    };
    return BST;
}());

var useTreeState = function (treeObject) {
    var _a = React.useState(treeObject), tree = _a[0], setTreeInternal = _a[1];
    var _b = React.useState(treeObject === null || treeObject === void 0 ? void 0 : treeObject.root.currentJSX), treeJSX = _b[0], setTreeJSX = _b[1];
    var setTree = function (newTreeObject) {
        var _a;
        setTreeInternal(newTreeObject);
        setTreeJSX((_a = newTreeObject === null || newTreeObject === void 0 ? void 0 : newTreeObject.root) === null || _a === void 0 ? void 0 : _a.currentJSX);
    };
    return { tree: tree, treeJSX: treeJSX, setTree: setTree };
};

var useTreeStyle = function (treeStyles) {
    var handleStyles = function () {
        if (!treeStyles)
            return;
        for (var _i = 0, _a = Object.entries(treeStyles); _i < _a.length; _i++) {
            var _b = _a[_i], style = _b[0], value = _b[1];
            document
                .getElementById(TREE_ID)
                .style.setProperty(CSS_VARIABLE_MAP[style].variableName, value);
        }
    };
    React.useEffect(function () {
        handleStyles();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [treeStyles]);
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = "#react-tree-vis {\n  --line-color: #ccc;\n  --line-hover-color: #5f6674;\n  --line-radius: 5px;\n\n  --node-border: none;\n  --node-border-radius: 200px;\n  --node-bg-color: #fff;\n  --node-font-color: #666;\n  --node-shadow: -5px -5px 20px #fff, 5px 5px 20px #babecc;\n  --node-text-shadow: none;\n\n  /* highlighted node */\n  --node-hl-border: none;\n  --node-hl-bg-color: #fff;\n  --node-hl-font-color: #fff;\n  --node-hl-shadow: -5px -5px 20px #fff, 5px 5px 20px #babecc;\n  --node-hl-text-shadow: 0 0 5px #fff, 0 0 10px #fff, 0 0 15px #00ff15,\n    0 0 20px #00ff15, 0 0 25px #00ff15, 0 0 30px #00ff15, 0 0 35px #00ff15;\n\n    --node-hover-border: none;\n  --node-hover-bg-color: #fff;\n  --node-hover-font-color: #002574;\n  --node-hover-shadow: -1px -1px 5px #fff, 1px 1px 5px #babecc;\n  --node-hover-text-shadow: none;\n\n  --node-null-font-color: #7c7c7c2f;\n  --node-null-hover-font-color: #ff0000b9;\n\n  --transition: 0.5s;\n}\n\n#react-tree-vis ul {\n  padding: 0;\n  padding-top: 20px;\n  position: relative;\n  transition: all var(--transition);\n}\n\n#react-tree-vis > ul {\n  width: fit-content;\n  margin: 0 auto;\n}\n\n#react-tree-vis li {\n  float: left;\n  text-align: center;\n  list-style-type: none;\n  position: relative;\n  padding: 10px 1px 0 1px;\n  transition: all var(--transition);\n}\n\n/* Line Styles */\n#react-tree-vis li::before,\n#react-tree-vis li::after {\n  content: '';\n  position: absolute;\n  top: 0;\n  right: 50%;\n  border-top: 1px solid var(--line-color);\n  width: 50%;\n  height: 10px;\n  transition: all var(--transition);\n}\n\n#react-tree-vis li::after {\n  right: auto;\n  left: 50%;\n  border-left: 1px solid var(--line-color);\n}\n\n#react-tree-vis li:last-child::before {\n  border-right: 1px solid var(--line-color);\n  border-radius: 0 var(--line-radius) 0 0;\n}\n\n#react-tree-vis li:first-child::after {\n  border-radius: var(--line-radius) 0 0 0;\n}\n\n#react-tree-vis ul ul::before {\n  content: '';\n  position: absolute;\n  top: 0;\n  left: 50%;\n  border-left: 1px solid var(--line-color);\n  width: 0;\n  height: 20px;\n  transition: all var(--transition);\n}\n\n#react-tree-vis li:only-child::after,\n#react-tree-vis li:only-child::before {\n  display: none;\n}\n\n#react-tree-vis li:only-child {\n  padding-top: 0;\n}\n\n#react-tree-vis li:first-child::before,\n#react-tree-vis li:last-child::after {\n  border: 0 none;\n}\n\n/* Node Styles starts */\n#react-tree-vis li div {\n  padding: 6px 12px;\n  text-decoration: none;\n  color: var(--node-font-color);\n  font-family: arial, verdana, tahoma;\n  font-size: 20px;\n  display: inline-block;\n  box-shadow: var(--node-shadow);\n  text-shadow: var(--node-text-shadow);\n  border-radius: var(--node-border-radius);\n  border: var(--node-border);\n  background-color: var(--node-bg-color);\n  transition: all var(--transition);\n}\n\n#react-tree-vis li div.end {\n  color: #c20000;\n}\n\n#react-tree-vis li div.red {\n  background-color: rgba(218, 96, 96, 0.5);\n  text-shadow: none;\n}\n\n#react-tree-vis li div.black {\n  background-color: rgba(172, 172, 172, 0.5);\n  text-shadow: none;\n}\n\n#react-tree-vis li div.highlight {\n  background-color: var(--node-hl-bg-color);\n  border: var(--node-hl-border);\n  box-shadow: var(--node-hl-shadow);\n  color: var(--node-hl-font-color);\n  text-shadow: var(--node-hl-text-shadow);\n}\n\n#react-tree-vis li div.highlight.red {\n  color: #fff;\n  text-shadow: 0 0 5px #fff, 0 0 10px #fff, 0 0 15px #f00, 0 0 20px #f00,\n    0 0 25px #f00, 0 0 30px #f00, 0 0 35px #f00;\n}\n\n#react-tree-vis li div.highlight.black {\n  color: #fff;\n  text-shadow: 0 0 5px #fff, 0 0 10px #fff, 0 0 15px #000, 0 0 20px #000,\n    0 0 25px #000, 0 0 30px #000, 0 0 35px #000;\n}\n\n/* Hover Styles starts */\n#react-tree-vis li div:hover {\n  border: var(--node-hover-border);\n  color: var(--node-hover-font-color);\n  background-color: var(--node-hover-bg-color);\n  box-shadow: var(--node-hover-shadow);\n  text-shadow: var(--node-hover-text-shadow);\n}\n\n#react-tree-vis li div:hover + ul li::after,\n#react-tree-vis li div:hover + ul li::before,\n#react-tree-vis li div:hover + ul::before,\n#react-tree-vis li div:hover + ul ul::before {\n  border-color: var(--line-hover-color);\n}\n\n#react-tree-vis li.null div {\n  color: var(--node-null-font-color);\n}\n\n#react-tree-vis li.null div:hover {\n  color: var(--node-null-hover-font-color);\n}\n\n#react-tree-vis .height {\n  font-size: 13px;\n}\n";
styleInject(css_248z);

var BinarySearchTree = function (_a, ref) {
    var data = _a.data, treeStyles = _a.treeStyles;
    var _b = useTreeState(null), tree = _b.tree, treeJSX = _b.treeJSX, setTree = _b.setTree;
    useTreeStyle(treeStyles);
    React.useImperativeHandle(ref, function () { return ({
        insert: function (value) {
            // if tree is empty
            if (!(tree === null || tree === void 0 ? void 0 : tree.root)) {
                var newTree = new BST();
                newTree.insert(value);
                setTree(newTree);
                return;
            }
            tree.insert(value);
            setTree(tree);
        },
        remove: function (value) {
            if (!(tree === null || tree === void 0 ? void 0 : tree.root))
                return false;
            if (!tree.search(value))
                return false;
            tree.remove(value);
            setTree(tree.root ? tree : null);
            return true;
        },
        search: function (value) {
            if (!(tree === null || tree === void 0 ? void 0 : tree.root))
                return false;
            var foundNode = tree.search(value);
            if (foundNode)
                setTree(tree);
            return foundNode;
        },
        getData: function (traversalOrder) {
            var traversalData = [];
            tree[traversalOrder](traversalData);
            return traversalData;
        },
        clear: function () {
            setTree(new BST());
        },
        balance: function () {
            tree.balance();
            setTree(tree);
        },
        generateRandomTree: function (countOfNodes) {
            var newTree = new BST(countOfNodes);
            setTree(newTree);
        },
        checkTreeType: function () {
            return tree.checkBST();
        },
    }); });
    var handleData = function () {
        var newTree = new BST();
        data.forEach(function (elem) { return newTree.insert(elem); });
        setTree(newTree);
    };
    React.useEffect(function () {
        if (data)
            handleData();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [data]);
    return (React__default['default'].createElement("div", { id: TREE_ID, ref: ref },
        React__default['default'].createElement("ul", null, treeJSX)));
};
var BinarySearchTree$1 = React.forwardRef(BinarySearchTree);

var useTree = function () {
    var ref = React.useRef(null);
    var _a = React.useState(null), insertFunction = _a[0], setInsertFunction = _a[1];
    var _b = React.useState(null), removeFunction = _b[0], setRemoveFunction = _b[1];
    var _c = React.useState(null), searchFunction = _c[0], setSearchFunction = _c[1];
    var _d = React.useState(null), getDataFunction = _d[0], setGetDataFunction = _d[1];
    var _e = React.useState(null), clearFunction = _e[0], setClearFunction = _e[1];
    var _f = React.useState(null), balanceFunction = _f[0], setBalanceFunction = _f[1];
    var _g = React.useState(null), generateRandomTreeFunction = _g[0], setGenerateRandomTreeFunction = _g[1];
    var _h = React.useState(null), checkTreeTypeFunction = _h[0], setCheckTreeTypeFunction = _h[1];
    React.useEffect(function () {
        setInsertFunction(function () { return function (value) { var _a; return (_a = ref === null || ref === void 0 ? void 0 : ref.current) === null || _a === void 0 ? void 0 : _a.insert(value); }; });
        setRemoveFunction(function () { return function (value) { var _a; return (_a = ref === null || ref === void 0 ? void 0 : ref.current) === null || _a === void 0 ? void 0 : _a.remove(value); }; });
        setSearchFunction(function () { return function (value) { var _a; return (_a = ref === null || ref === void 0 ? void 0 : ref.current) === null || _a === void 0 ? void 0 : _a.search(value); }; });
        setGetDataFunction(function () { return function (traversalOrder) { var _a; return (_a = ref === null || ref === void 0 ? void 0 : ref.current) === null || _a === void 0 ? void 0 : _a.getData(traversalOrder); }; });
        setClearFunction(function () { return function () { var _a; return (_a = ref === null || ref === void 0 ? void 0 : ref.current) === null || _a === void 0 ? void 0 : _a.clear(); }; });
        setBalanceFunction(function () { return function () { var _a; return (_a = ref === null || ref === void 0 ? void 0 : ref.current) === null || _a === void 0 ? void 0 : _a.balance(); }; });
        setGenerateRandomTreeFunction(function () { return function (countOfNodes) { var _a; return (_a = ref === null || ref === void 0 ? void 0 : ref.current) === null || _a === void 0 ? void 0 : _a.generateRandomTree(countOfNodes); }; });
        setCheckTreeTypeFunction(function () { return function () { var _a; return (_a = ref === null || ref === void 0 ? void 0 : ref.current) === null || _a === void 0 ? void 0 : _a.checkTreeType(); }; });
    }, [ref]);
    return {
        ref: ref,
        insert: insertFunction,
        remove: removeFunction,
        search: searchFunction,
        getData: getDataFunction,
        clear: clearFunction,
        balance: balanceFunction,
        generateRandomTree: generateRandomTreeFunction,
        checkTreeType: checkTreeTypeFunction,
    };
};

exports.BinarySearchTree = BinarySearchTree$1;
exports.useTree = useTree;
//# sourceMappingURL=index.js.map
